@@ -6,6 +6,7 @@ var _dotenv = _interopRequireDefault(require("dotenv"));
 var _electron = require("electron");
 var _findProcess = _interopRequireDefault(require("find-process"));
 var fsPromises = _interopRequireWildcard(require("fs/promises"));
+var net = _interopRequireWildcard(require("net"));
 var _nodeFs = _interopRequireDefault(require("node:fs"));
 var _nodeOs = require("node:os");
 var _nodeUtil = require("node:util");
@@ -16,15 +17,35 @@ var _yargs = _interopRequireDefault(require("yargs"));
 var _helpers = require("yargs/helpers");
 var _i18next = _interopRequireDefault(require("./i18next.config"));
 var _pluginManagement = require("./plugin-management");
+var _runCmd = require("./runCmd");
 var _windowSize = _interopRequireDefault(require("./windowSize"));
 function _getRequireWildcardCache(e) { if ("function" != typeof WeakMap) return null; var r = new WeakMap(), t = new WeakMap(); return (_getRequireWildcardCache = function (e) { return e ? t : r; })(e); }
 function _interopRequireWildcard(e, r) { if (!r && e && e.__esModule) return e; if (null === e || "object" != typeof e && "function" != typeof e) return { default: e }; var t = _getRequireWildcardCache(r); if (t && t.has(e)) return t.get(e); var n = { __proto__: null }, a = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var u in e) if ("default" !== u && {}.hasOwnProperty.call(e, u)) { var i = a ? Object.getOwnPropertyDescriptor(e, u) : null; i && (i.get || i.set) ? Object.defineProperty(n, u, i) : n[u] = e[u]; } return n.default = e, t && t.set(e, n), n; }
 function _interopRequireDefault(e) { return e && e.__esModule ? e : { default: e }; }
+/*
+ * Copyright 2025 The Kubernetes Authors
+ *
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ *
+ * http://www.apache.org/licenses/LICENSE-2.0
+ *
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
+
+let isRunningScript = false;
+if (process.env.HEADLAMP_RUN_SCRIPT) {
+  isRunningScript = true;
+  (0, _runCmd.runScript)();
+}
 _dotenv.default.config({
   path: _path.default.join(process.resourcesPath, '.env')
 });
-const pathInfoDebug = false;
-let pathInfo;
 const isDev = process.env.ELECTRON_DEV || false;
 let frontendPath = '';
 if (isDev) {
@@ -62,6 +83,15 @@ const args = (0, _yargs.default)((0, _helpers.hideBin)(process.argv)).command('l
   'disable-gpu': {
     describe: 'Disable use of GPU. For people who may have buggy graphics drivers',
     type: 'boolean'
+  },
+  'watch-plugins-changes': {
+    describe: 'Reloads plugins when there are changes to them or their directory',
+    type: 'boolean'
+  },
+  port: {
+    describe: 'Port for the backend server to listen on',
+    type: 'number',
+    default: 4466
   }
 }).positional('kubeconfig', {
   describe: 'Path to the kube config file (uses the default kube config location if not specified)',
@@ -69,12 +99,12 @@ const args = (0, _yargs.default)((0, _helpers.hideBin)(process.argv)).command('l
 }).help().parseSync();
 const isHeadlessMode = args.headless === true;
 let disableGPU = args['disable-gpu'] === true;
-const defaultPort = 4466;
+const defaultPort = args.port || 4466;
+let actualPort = defaultPort; // Will be updated when backend starts
+const MAX_PORT_ATTEMPTS = Math.abs(Number(process.env.HEADLAMP_MAX_PORT_ATTEMPTS) || 100); // Maximum number of ports to try
+
 const useExternalServer = process.env.EXTERNAL_SERVER || false;
 const shouldCheckForUpdates = process.env.HEADLAMP_CHECK_FOR_UPDATES !== 'false';
-const manifestDir = isDev ? _path.default.resolve('./') : process.resourcesPath;
-const manifestFile = _path.default.join(manifestDir, 'app-build-manifest.json');
-const buildManifest = _nodeFs.default.existsSync(manifestFile) ? require(manifestFile) : {};
 
 // make it global so that it doesn't get garbage collected
 let mainWindow;
@@ -212,7 +242,7 @@ class PluginManagerEventListeners {
       percentage: 0,
       controller
     };
-    let pluginInfo;
+    let pluginInfo = undefined;
     try {
       pluginInfo = await _pluginManagement.PluginManager.fetchPluginInfo(URL, {
         signal: controller.signal
@@ -227,13 +257,22 @@ class PluginManagerEventListeners {
         message: 'Failed to fetch plugin info'
       };
     }
+    const {
+      matchingExtraFiles
+    } = (0, _pluginManagement.getMatchingExtraFiles)(pluginInfo?.extraFiles ? pluginInfo?.extraFiles : {});
+    const extraUrls = matchingExtraFiles.map(file => file.url);
+    const allUrls = [pluginInfo.archiveURL, ...extraUrls].join(', ');
     const dialogOptions = {
       type: 'question',
-      buttons: ['Yes', 'No'],
+      buttons: [_i18next.default.t('Yes'), _i18next.default.t('No')],
       defaultId: 1,
-      title: 'Plugin Installation',
-      message: 'Do you want to install this plugin?',
-      detail: `You are about to install ${pluginName} plugin from: ${pluginInfo.archiveURL}\nDo you want to proceed?`
+      title: _i18next.default.t('Plugin Installation'),
+      message: _i18next.default.t('Do you want to install the plugin "{{ pluginName }}"?', {
+        pluginName
+      }),
+      detail: _i18next.default.t('You are about to install a plugin from: {{ url }}\nDo you want to proceed?', {
+        url: allUrls
+      })
     };
     let userChoice;
     try {
@@ -274,6 +313,7 @@ class PluginManagerEventListeners {
       percentage: 10,
       controller
     };
+    (0, _runCmd.addRunCmdConsent)(pluginInfo);
     _pluginManagement.PluginManager.installFromPluginPkg(pluginInfo, destinationFolder, headlampVersion, progress => {
       updateCache(progress);
     }, controller.signal);
@@ -351,6 +391,7 @@ class PluginManagerEventListeners {
         message: 'uninstalling plugin'
       }
     };
+    (0, _runCmd.removeRunCmdConsent)(pluginName);
     _pluginManagement.PluginManager.uninstall(pluginName, destinationFolder, progress => {
       updateCache(progress);
     });
@@ -359,6 +400,9 @@ class PluginManagerEventListeners {
   /**
    * Handles the list event.
    *
+   * Lists plugins from all three directories (shipped, user-installed, development)
+   * and returns a combined list with their locations.
+   *
    * @method
    * @name handleList
    * @param {Electron.IpcMainEvent} event - The IPC Main Event.
@@ -370,12 +414,78 @@ class PluginManagerEventListeners {
       identifier,
       destinationFolder
     } = eventData;
-    _pluginManagement.PluginManager.list(destinationFolder, progress => {
+
+    // If a specific folder is requested, list only from that folder
+    if (destinationFolder) {
+      _pluginManagement.PluginManager.list(destinationFolder, progress => {
+        event.sender.send('plugin-manager', JSON.stringify({
+          identifier: identifier,
+          ...progress
+        }));
+      });
+      return;
+    }
+
+    // Otherwise, list from all three directories
+    try {
+      const allPlugins = [];
+
+      // List from shipped plugins (.plugins)
+      const shippedDir = _path.default.join(__dirname, '.plugins');
+      try {
+        const shippedPlugins = _pluginManagement.PluginManager.list(shippedDir);
+        if (shippedPlugins) {
+          allPlugins.push(...shippedPlugins);
+        }
+      } catch (error) {
+        // Only ignore if directory doesn't exist, log other errors
+        if (error?.code !== 'ENOENT') {
+          console.error('Error listing shipped plugins:', error);
+        }
+      }
+
+      // List from user-installed plugins (user-plugins)
+      const userDir = (0, _pluginManagement.defaultUserPluginsDir)();
+      try {
+        const userPlugins = _pluginManagement.PluginManager.list(userDir);
+        if (userPlugins) {
+          allPlugins.push(...userPlugins);
+        }
+      } catch (error) {
+        // Only ignore if directory doesn't exist, log other errors
+        if (error?.code !== 'ENOENT') {
+          console.error('Error listing user plugins:', error);
+        }
+      }
+
+      // List from development plugins (plugins)
+      const devDir = (0, _pluginManagement.defaultPluginsDir)();
+      try {
+        const devPlugins = _pluginManagement.PluginManager.list(devDir);
+        if (devPlugins) {
+          allPlugins.push(...devPlugins);
+        }
+      } catch (error) {
+        // Only ignore if directory doesn't exist, log other errors
+        if (error?.code !== 'ENOENT') {
+          console.error('Error listing development plugins:', error);
+        }
+      }
+
+      // Send combined results
       event.sender.send('plugin-manager', JSON.stringify({
         identifier: identifier,
-        ...progress
+        type: 'success',
+        message: 'Plugins Listed',
+        data: allPlugins
       }));
-    });
+    } catch (error) {
+      event.sender.send('plugin-manager', JSON.stringify({
+        identifier: identifier,
+        type: 'error',
+        message: error instanceof Error ? error.message : String(error)
+      }));
+    }
   }
 
   /**
@@ -522,20 +632,81 @@ async function getShellEnv() {
     return process.env;
   }
 }
+
+/**
+ * Check if a port is available by attempting to create a server on it
+ */
+async function isPortAvailable(port) {
+  return new Promise(resolve => {
+    const server = net.createServer();
+    server.once('error', () => {
+      resolve(false);
+    });
+    server.once('listening', () => {
+      server.close(() => resolve(true));
+    });
+    try {
+      server.listen({
+        port,
+        host: 'localhost',
+        exclusive: true
+      });
+    } catch (err) {
+      server.emit('error', err);
+    }
+  });
+}
+
+/**
+ * Find an available port starting from the default port
+ * Tries to find a free port, skipping all occupied ports (including Headlamp)
+ * @returns Available port number, or throws if no port found after MAX_PORT_ATTEMPTS
+ */
+async function findAvailablePort(startPort) {
+  for (let i = 0; i < MAX_PORT_ATTEMPTS; i++) {
+    const port = startPort + i;
+    // Skip ports already used by another Headlamp instance.
+    const headlampPIDs = await getHeadlampPIDsOnPort(port);
+    if (headlampPIDs && headlampPIDs.length > 0) {
+      console.info(`Port ${port} is occupied by Headlamp process(es) ${headlampPIDs.join(', ')}, trying next port...`);
+      continue;
+    }
+    const available = await isPortAvailable(port);
+    if (available) {
+      if (port !== startPort) {
+        console.info(`Port ${startPort} is in use, using port ${port} instead`);
+      }
+      return port;
+    }
+    console.info(`Port ${port} is occupied by another process, trying next port...`);
+  }
+  throw new Error(`Could not find an available port after ${MAX_PORT_ATTEMPTS} attempts starting from ${startPort}`);
+}
 async function startServer(flags = []) {
   const serverFilePath = isDev ? _path.default.resolve('../backend/headlamp-server') : _path.default.join(process.resourcesPath, './headlamp-server');
-  let serverArgs = [];
+  actualPort = await findAvailablePort(defaultPort);
+  let serverArgs = ['--listen-addr=localhost', `--port=${actualPort}`];
   if (!!args.kubeconfig) {
     serverArgs = serverArgs.concat(['--kubeconfig', args.kubeconfig]);
   }
+  const manifestDir = isDev ? _path.default.resolve('./') : process.resourcesPath;
+  const manifestFile = _path.default.join(manifestDir, 'app-build-manifest.json');
+  let buildManifest = {};
+  try {
+    const manifestContent = await fsPromises.readFile(manifestFile, 'utf8');
+    buildManifest = JSON.parse(manifestContent);
+  } catch (err) {
+    // If the manifest doesn't exist or can't be read, fall back to empty object
+    buildManifest = {};
+  }
   const proxyUrls = !!buildManifest && buildManifest['proxy-urls'];
   if (!!proxyUrls && proxyUrls.length > 0) {
     serverArgs = serverArgs.concat(['--proxy-urls', proxyUrls.join(',')]);
   }
-  const bundledPlugins = _path.default.join(process.resourcesPath, '.plugins');
-  function isEmpty(path) {
-    return _nodeFs.default.readdirSync(path).length === 0;
+  if (args.watchPluginsChanges !== undefined) {
+    serverArgs.push(`--watch-plugins-changes=${args.watchPluginsChanges}`);
   }
+  const bundledPlugins = _path.default.join(process.resourcesPath, '.plugins');
 
   // Enable the Helm and dynamic cluster endpoints
   process.env.HEADLAMP_CONFIG_ENABLE_HELM = 'true';
@@ -545,8 +716,16 @@ async function startServer(flags = []) {
   process.env.HEADLAMP_BACKEND_TOKEN = backendToken;
 
   // Set the bundled plugins in addition to the the user's plugins.
-  if (_nodeFs.default.existsSync(bundledPlugins) && !isEmpty(bundledPlugins)) {
-    process.env.HEADLAMP_STATIC_PLUGINS_DIR = bundledPlugins;
+  try {
+    const stat = await fsPromises.stat(bundledPlugins);
+    if (stat.isDirectory()) {
+      const entries = await fsPromises.readdir(bundledPlugins);
+      if (entries.length !== 0) {
+        process.env.HEADLAMP_STATIC_PLUGINS_DIR = bundledPlugins;
+      }
+    }
+  } catch (err) {
+    // Directory doesn't exist or is not readable — ignore and continue.
   }
   serverArgs = serverArgs.concat(flags);
   console.log('arguments passed to backend server', serverArgs);
@@ -571,13 +750,17 @@ async function startServer(flags = []) {
  * Are we running inside WSL?
  * @returns true if we are running inside WSL.
  */
-function isWSL() {
+async function isWSL() {
+  // Cheap platform check first to avoid reading /proc on non-Linux platforms.
+  if ((0, _os.platform)() !== 'linux') {
+    return false;
+  }
   try {
-    const data = _nodeFs.default.readFileSync('/proc/version', {
-      encoding: 'utf8',
-      flag: 'r'
+    const data = await fsPromises.readFile('/proc/version', {
+      encoding: 'utf8'
     });
-    return data.indexOf('icrosoft') !== -1;
+    const lower = data.toLowerCase();
+    return lower.includes('microsoft') || lower.includes('wsl');
   } catch {
     return false;
   }
@@ -713,16 +896,19 @@ function getDefaultAppMenu() {
       id: 'original-toggle-dev-tools'
     }, sep, {
       label: _i18next.default.t('Reset Zoom'),
-      role: 'resetzoom',
-      id: 'original-reset-zoom'
+      id: 'original-reset-zoom',
+      accelerator: 'CmdOrCtrl+0',
+      click: () => setZoom(1.0)
     }, {
       label: _i18next.default.t('Zoom In'),
-      role: 'zoomin',
-      id: 'original-zoom-in'
+      id: 'original-zoom-in',
+      accelerator: 'CmdOrCtrl+Plus',
+      click: () => adjustZoom(0.1)
     }, {
       label: _i18next.default.t('Zoom Out'),
-      role: 'zoomout',
-      id: 'original-zoom-out'
+      id: 'original-zoom-out',
+      accelerator: 'CmdOrCtrl+-',
+      click: () => adjustZoom(-0.1)
     }, sep, {
       label: _i18next.default.t('Toggle Fullscreen'),
       role: 'togglefullscreen',
@@ -793,11 +979,10 @@ function getDefaultAppMenu() {
     }, {
       label: _i18next.default.t('Open an Issue'),
       id: 'original-open-issue',
-      url: 'https://github.com/headlamp-k8s/headlamp/issues'
+      url: 'https://github.com/kubernetes-sigs/headlamp/issues'
     }, {
       label: _i18next.default.t('About'),
-      id: 'original-about',
-      url: 'https://github.com/headlamp-k8s/headlamp'
+      id: 'original-about-help'
     }]
   }];
   return appMenu;
@@ -870,7 +1055,13 @@ function menusToTemplate(mainWindow, menusFromPlugins) {
     if (!loadFullMenu && !!afterPlugins) {
       return;
     }
-    if (!!url) {
+
+    // Handle the "About" menu item from the Help menu specially
+    if (appMenu.id === 'original-about-help') {
+      menu.click = () => {
+        mainWindow?.webContents.send('open-about-dialog');
+      };
+    } else if (!!url) {
       menu.click = async () => {
         // Open external links in the external browser.
         if (!!mainWindow && !url.startsWith('http')) {
@@ -896,6 +1087,46 @@ async function getRunningHeadlampPIDs() {
   }
   return processes.map(pInfo => pInfo.pid);
 }
+
+/**
+ * Check if a specific port is occupied by a Headlamp process
+ * @returns Array of Headlamp PIDs using the port, or null if port is free or used by another process
+ */
+async function getHeadlampPIDsOnPort(port) {
+  try {
+    // Get all Headlamp processes
+    const headlampProcesses = await (0, _findProcess.default)('name', 'headlamp-server');
+    if (headlampProcesses.length === 0) {
+      return null;
+    }
+
+    // Parse command line arguments to find which Headlamp process is using this port
+    const headlampOnPort = headlampProcesses.filter(p => {
+      if (!p.cmd) return false;
+
+      // Look for --port=XXXX or --port XXXX in the command line
+      const portRegex = /--port[=\s]+(\d+)/;
+      const match = p.cmd.match(portRegex);
+      if (match && match[1]) {
+        const processPort = parseInt(match[1], 10);
+        return processPort === port;
+      }
+
+      // If no port specified, Headlamp uses default port 4466
+      if (port === 4466 && !p.cmd.includes('--port')) {
+        return true;
+      }
+      return false;
+    });
+    if (headlampOnPort.length === 0) {
+      return null;
+    }
+    return headlampOnPort.map(p => p.pid);
+  } catch (error) {
+    console.error(`Error checking if port ${port} is used by Headlamp:`, error);
+    return null;
+  }
+}
 function killProcess(pid) {
   if (process.platform === 'win32') {
     // Otherwise on Windows the process will stick around.
@@ -904,6 +1135,42 @@ function killProcess(pid) {
     process.kill(pid, 'SIGHUP');
   }
 }
+const ZOOM_FILE_PATH = _path.default.join(_electron.app.getPath('userData'), 'headlamp-config.json');
+let cachedZoom = 1.0;
+function saveZoomFactor(factor) {
+  try {
+    _nodeFs.default.writeFileSync(ZOOM_FILE_PATH, JSON.stringify({
+      zoomFactor: factor
+    }), 'utf-8');
+  } catch (err) {
+    console.error('Failed to save zoom factor:', err);
+  }
+}
+async function loadZoomFactor() {
+  try {
+    const content = await fsPromises.readFile(ZOOM_FILE_PATH, 'utf-8');
+    const {
+      zoomFactor = 1.0
+    } = JSON.parse(content);
+    return typeof zoomFactor === 'number' ? zoomFactor : 1.0;
+  } catch (err) {
+    console.error('Failed to load zoom factor, defaulting to 1.0:', err);
+    return 1.0;
+  }
+}
+
+// The zoom factor should respect the fixed limits set by Electron.
+function clampZoom(factor) {
+  return Math.min(5.0, Math.max(0.25, factor));
+}
+function setZoom(factor) {
+  cachedZoom = factor;
+  mainWindow?.webContents.setZoomFactor(cachedZoom);
+}
+function adjustZoom(delta) {
+  const newZoom = clampZoom(cachedZoom + delta);
+  setZoom(newZoom);
+}
 function startElecron() {
   console.info('App starting...');
   let appVersion;
@@ -914,9 +1181,129 @@ function startElecron() {
     appVersion = _electron.app.getVersion();
   }
   console.log('Check for updates: ', shouldCheckForUpdates);
+  async function startServerIfNeeded() {
+    if (!useExternalServer) {
+      try {
+        // Try to start the server (it will find an available port)
+        serverProcess = await startServer();
+        attachServerEventHandlers(serverProcess);
+        serverProcess.addListener('exit', async e => {
+          const ERROR_ADDRESS_IN_USE = 98;
+          if (e === ERROR_ADDRESS_IN_USE) {
+            // This is a fallback - we should have already checked for port conflicts
+            // before starting the server. This handles edge cases where the port
+            // became occupied between our check and the server start.
+            console.warn('Server failed to start due to address in use (unexpected)');
+            const runningHeadlamp = await getRunningHeadlampPIDs();
+            if (!mainWindow) {
+              return;
+            }
+            if (!!runningHeadlamp) {
+              const resp = _electron.dialog.showMessageBoxSync(mainWindow, {
+                title: _i18next.default.t('Another process is running'),
+                message: _i18next.default.t('Looks like another process is already running. Continue by terminating that process automatically, or quit?'),
+                type: 'question',
+                buttons: [_i18next.default.t('Continue'), _i18next.default.t('Quit')]
+              });
+              if (resp === 0) {
+                runningHeadlamp.forEach(pid => {
+                  try {
+                    killProcess(pid);
+                  } catch (e) {
+                    const message = e instanceof Error ? e.message : String(e);
+                    console.error(`Failed to kill process with PID ${pid}: ${message}`);
+                  }
+                });
+
+                // Wait a bit and retry
+                await new Promise(resolve => setTimeout(resolve, 1000));
+              } else {
+                mainWindow.close();
+                return;
+              }
+            }
+
+            // If we couldn't kill the process, warn the user and quit.
+            const processes = await getRunningHeadlampPIDs();
+            if (!!processes) {
+              _electron.dialog.showMessageBoxSync({
+                type: 'warning',
+                title: _i18next.default.t('Failed to quit the other running process'),
+                message: _i18next.default.t(`Could not quit the other running process, PIDs: {{ process_list }}. Please stop that process and relaunch the app.`, {
+                  process_list: processes
+                })
+              });
+              mainWindow.close();
+              return;
+            }
+            serverProcess = await startServer();
+            attachServerEventHandlers(serverProcess);
+          }
+        });
+      } catch (error) {
+        // Failed to find an available port after all attempts
+        const message = error instanceof Error ? error.message : String(error);
+        console.error('Failed to find an available port:', message);
+        if (!mainWindow) {
+          console.error('Cannot show dialog - no main window available');
+          return;
+        }
+
+        // Ask user if they want to kill existing Headlamp processes and retry
+        const headlampPIDs = await getRunningHeadlampPIDs();
+        if (headlampPIDs && headlampPIDs.length > 0) {
+          const resp = _electron.dialog.showMessageBoxSync(mainWindow, {
+            title: _i18next.default.t('No available ports'),
+            message: _i18next.default.t('Could not find an available port. There are processes running on ports {{startPort}}-{{endPort}}. Terminate these processes and retry?', {
+              startPort: defaultPort,
+              endPort: defaultPort + MAX_PORT_ATTEMPTS - 1
+            }),
+            type: 'warning',
+            buttons: [_i18next.default.t('Terminate and Retry'), _i18next.default.t('Quit')]
+          });
+          if (resp === 0) {
+            // User chose to terminate and retry
+            console.info(`Terminating ${headlampPIDs.length} Headlamp process(es)...`);
+            headlampPIDs.forEach(pid => {
+              try {
+                killProcess(pid);
+              } catch (e) {
+                const killMessage = e instanceof Error ? e.message : String(e);
+                console.error(`Failed to kill headlamp-server process with PID ${pid}: ${killMessage}`);
+              }
+            });
+
+            // Wait for processes to be killed
+            await new Promise(resolve => setTimeout(resolve, 1000));
+
+            // Retry starting the server
+            try {
+              serverProcess = await startServer();
+              attachServerEventHandlers(serverProcess);
+            } catch (retryError) {
+              const retryMessage = retryError instanceof Error ? retryError.message : String(retryError);
+              console.error('Failed to start server after killing processes:', retryMessage);
+              _electron.dialog.showErrorBox(_i18next.default.t('Failed to start'), _i18next.default.t('Could not start the server even after terminating existing processes.'));
+              mainWindow.close();
+            }
+          } else {
+            // User chose to quit
+            mainWindow.close();
+          }
+        } else {
+          // No Headlamp processes found, but still can't find a port
+          _electron.dialog.showErrorBox(_i18next.default.t('No available ports'), _i18next.default.t('Could not find an available port in the range {{startPort}}-{{endPort}}. Please free up a port and try again.', {
+            startPort: defaultPort,
+            endPort: defaultPort + MAX_PORT_ATTEMPTS - 1
+          }));
+          mainWindow.close();
+        }
+      }
+    }
+  }
   async function createWindow() {
     // WSL has a problem with full size window placement, so make it smaller.
-    const withMargin = isWSL();
+    const withMargin = await isWSL();
     const {
       width,
       height
@@ -930,6 +1317,9 @@ function startElecron() {
         preload: `${__dirname}/preload.js`
       }
     });
+
+    // Load the frontend
+    mainWindow.loadURL(startUrl);
     setMenu(mainWindow, currentMenu);
     mainWindow.webContents.setWindowOpenHandler(({
       url
@@ -957,6 +1347,15 @@ function startElecron() {
         goForwardMenu.enabled = mainWindow?.webContents.canGoForward() || false;
       }
     });
+    mainWindow.webContents.on('did-finish-load', async () => {
+      const startZoom = await loadZoomFactor();
+      if (startZoom !== 1.0) {
+        setZoom(startZoom);
+      }
+
+      // Inject the backend port into the window object
+      mainWindow?.webContents.executeJavaScript(`window.headlampBackendPort = ${actualPort};`);
+    });
     mainWindow.webContents.on('dom-ready', () => {
       const defaultMenu = getDefaultAppMenu();
       const currentMenu = JSON.parse(JSON.stringify(defaultMenu));
@@ -966,6 +1365,21 @@ function startElecron() {
       mainWindow = null;
     });
 
+    // Workaround to cookies to be saved, since file:// protocal and localhost:port
+    // are treated as a cross site request.
+    mainWindow.webContents.session.webRequest.onHeadersReceived((details, callback) => {
+      if (details.url.startsWith(`http://localhost:${actualPort}`)) {
+        callback({
+          responseHeaders: {
+            ...details.responseHeaders,
+            'Set-Cookie': details.responseHeaders?.['Set-Cookie']?.map(it => it.replace('SameSite=Strict', 'SameSite=None;Secure=true')) ?? []
+          }
+        });
+      } else {
+        callback(details);
+      }
+    });
+
     // Force Single Instance Application
     const gotTheLock = _electron.app.requestSingleInstanceLock();
     if (gotTheLock) {
@@ -985,7 +1399,8 @@ function startElecron() {
     if a library is trying to open a url other than app url in electron take it
     to the default browser
     */
-    mainWindow.webContents.on('will-navigate', (event, url) => {
+    mainWindow.webContents.on('will-navigate', (event, encodedUrl) => {
+      const url = decodeURI(encodedUrl);
       if (url.startsWith(startUrl)) {
         return;
       }
@@ -1026,13 +1441,6 @@ function startElecron() {
       loadFullMenu = true;
       console.info('Plugins are loaded. Loading full menu.');
       setMenu(mainWindow, currentMenu);
-      if (pathInfoDebug && mainWindow) {
-        _electron.dialog.showMessageBoxSync(mainWindow, {
-          type: 'info',
-          title: 'Path debug info',
-          message: JSON.stringify(pathInfo)
-        });
-      }
     });
     _electron.ipcMain.on('setMenu', (event, menus) => {
       if (!mainWindow) {
@@ -1064,111 +1472,41 @@ function startElecron() {
     _electron.ipcMain.on('request-backend-token', () => {
       mainWindow?.webContents.send('backend-token', backendToken);
     });
-
-    /**
-     * Data sent from the renderer process when a 'run-command' event is emitted.
-     */
-
-    /**
-     * Handles 'run-command' events from the renderer process.
-     *
-     * Spawns the requested command and sends 'command-stdout',
-     * 'command-stderr', and 'command-exit' events back to the renderer
-     * process with the command's output and exit code.
-     *
-     * @param event - The event object.
-     * @param eventData - The data sent from the renderer process.
-     */
-    function handleRunCommand(event, eventData) {
-      return; // Disable this until we figure out a better way to do this
-
-      // Only allow "minikube", and "az" commands
-      const validCommands = ['minikube', 'az'];
-      if (!validCommands.includes(eventData.command)) {
-        console.error(`Invalid command: ${eventData.command}, only valid commands are: ${JSON.stringify(validCommands)}`);
-        return;
-      }
-      const child = (0, _child_process.spawn)(eventData.command, eventData.args, {
-        ...eventData.options,
-        shell: false
-      });
-      child.stdout.on('data', data => {
-        event.sender.send('command-stdout', eventData.id, data.toString());
-      });
-      child.stderr.on('data', data => {
-        event.sender.send('command-stderr', eventData.id, data.toString());
-      });
-      child.on('exit', code => {
-        event.sender.send('command-exit', eventData.id, code);
-      });
-    }
-    _electron.ipcMain.on('run-command', handleRunCommand);
+    _electron.ipcMain.on('request-backend-port', () => {
+      mainWindow?.webContents.send('backend-port', actualPort);
+    });
+    (0, _runCmd.setupRunCmdHandlers)(mainWindow, _electron.ipcMain);
     new PluginManagerEventListeners().setupEventHandlers();
-    if (!useExternalServer) {
-      const runningHeadlamp = await getRunningHeadlampPIDs();
-      let shouldWaitForKill = true;
-      if (!!runningHeadlamp) {
-        const resp = _electron.dialog.showMessageBoxSync(mainWindow, {
-          // Avoiding mentioning Headlamp here because it may run under a different name depending on branding (plugins).
-          title: _i18next.default.t('Another process is running'),
-          message: _i18next.default.t('Looks like another process is already running. Continue by terminating that process automatically, or quit?'),
-          type: 'question',
-          buttons: [_i18next.default.t('Continue'), _i18next.default.t('Quit')]
+
+    // Handle opening plugin folder in file explorer
+    _electron.ipcMain.on('open-plugin-folder', (event, pluginInfo) => {
+      let folderPath = null;
+      if (pluginInfo.type === 'user') {
+        folderPath = _path.default.join((0, _pluginManagement.defaultUserPluginsDir)(), pluginInfo.folderName);
+      } else if (pluginInfo.type === 'development') {
+        folderPath = _path.default.join((0, _pluginManagement.defaultPluginsDir)(), pluginInfo.folderName);
+      } else if (pluginInfo.type === 'shipped') {
+        folderPath = _path.default.join(process.resourcesPath, '.plugins', pluginInfo.folderName);
+      }
+      if (folderPath) {
+        _electron.shell.openPath(folderPath).catch(err => {
+          console.error('Failed to open plugin folder:', err);
         });
-        if (resp === 0) {
-          runningHeadlamp.forEach(pid => {
-            try {
-              killProcess(pid);
-            } catch (e) {
-              console.log(`Failed to quit headlamp-servere:`, e.message);
-              shouldWaitForKill = false;
-            }
-          });
-        } else {
-          mainWindow.close();
-          return;
-        }
       }
+    });
 
-      // If we reach here, then we attempted to kill headlamp-server. Let's make sure it's killed
-      // before starting our own, or else we may end up in a race condition (failing to start the
-      // new one before the existing one is fully killed).
-      if (!!runningHeadlamp && shouldWaitForKill) {
-        let stillRunning = true;
-        let timeWaited = 0;
-        const maxWaitTime = 3000; // ms
-        // @todo: Use an iterative back-off strategy for the wait (so we can start by waiting for shorter times).
-        for (let tries = 1; timeWaited < maxWaitTime && stillRunning; tries++) {
-          console.debug(`Checking if Headlamp is still running after we asked it to be killed; ${tries} ${timeWaited}/${maxWaitTime}ms wait.`);
-
-          // Wait (10 * powers of 2) ms with a max of 250 ms
-          const waitTime = Math.min(10 * tries ** 2, 250); // ms
-          await new Promise(f => setTimeout(f, waitTime));
-          timeWaited += waitTime;
-          stillRunning = !!(await getRunningHeadlampPIDs());
-          console.debug(stillRunning ? 'Still running...' : 'No longer running!');
-        }
-      }
-
-      // If we couldn't kill the process, warn the user and quit.
-      const processes = await getRunningHeadlampPIDs();
-      if (!!processes) {
-        _electron.dialog.showMessageBoxSync({
-          type: 'warning',
-          title: _i18next.default.t('Failed to quit the other running process'),
-          message: _i18next.default.t(`Could not quit the other running process, PIDs: {{ process_list }}. Please stop that process and relaunch the app.`, {
-            process_list: processes
-          })
-        });
-        mainWindow.close();
-        return;
-      }
-      serverProcess = await startServer();
-      attachServerEventHandlers(serverProcess);
+    // Also add bundled plugin bin directories to PATH
+    const bundledPlugins = _path.default.join(process.resourcesPath, '.plugins');
+    const bundledPluginBinDirs = (0, _pluginManagement.getPluginBinDirectories)(bundledPlugins);
+    if (bundledPluginBinDirs.length > 0) {
+      (0, _pluginManagement.addToPath)(bundledPluginBinDirs, 'bundled plugin');
     }
 
-    // Finally load the frontend
-    mainWindow.loadURL(startUrl);
+    // Add the installed plugins as well
+    const userPluginBinDirs = (0, _pluginManagement.getPluginBinDirectories)((0, _pluginManagement.defaultUserPluginsDir)());
+    if (userPluginBinDirs.length > 0) {
+      (0, _pluginManagement.addToPath)(userPluginBinDirs, 'userPluginBinDirs plugin');
+    }
   }
   if (disableGPU) {
     console.info('Disabling GPU hardware acceleration. Reason: related flag is set.');
@@ -1179,21 +1517,26 @@ function startElecron() {
   if (disableGPU) {
     _electron.app.disableHardwareAcceleration();
   }
-  _electron.app.on('ready', createWindow);
-  _electron.app.on('activate', function () {
+  _electron.app.on('ready', async () => {
+    await Promise.all([startServerIfNeeded(), createWindow()]);
+  });
+  _electron.app.on('activate', async function () {
     if (mainWindow === null) {
-      createWindow();
+      await Promise.all([startServerIfNeeded(), createWindow()]);
     }
   });
   _electron.app.once('window-all-closed', _electron.app.quit);
   _electron.app.once('before-quit', () => {
+    saveZoomFactor(cachedZoom);
     _i18next.default.off('languageChanged');
     if (mainWindow) {
       mainWindow.removeAllListeners('close');
     }
   });
 }
-_electron.app.on('quit', quitServerProcess);
+if (!isRunningScript) {
+  _electron.app.on('quit', quitServerProcess);
+}
 
 /**
  * add some error handlers to the serverProcess.
@@ -1203,8 +1546,22 @@ function attachServerEventHandlers(serverProcess) {
   serverProcess.on('error', err => {
     console.error(`server process failed to start: ${err}`);
   });
+  const extractPortFromOutput = data => {
+    const output = data.toString();
+    const portMatch = output.match(/Listen address:.*:(\d+)/);
+    if (portMatch && portMatch[1]) {
+      actualPort = parseInt(portMatch[1], 10);
+      console.info(`Backend server listening on port: ${actualPort}`);
+
+      // Update the environment variable for the frontend
+      if (mainWindow) {
+        mainWindow.webContents.executeJavaScript(`window.headlampBackendPort = ${actualPort};`);
+      }
+    }
+  };
   serverProcess.stdout.on('data', data => {
     console.info(`server process stdout: ${data}`);
+    extractPortFromOutput(data);
   });
   serverProcess.stderr.on('data', data => {
     const sterrMessage = `server process stderr: ${data}`;
@@ -1214,6 +1571,7 @@ function attachServerEventHandlers(serverProcess) {
     } else {
       console.error(sterrMessage);
     }
+    extractPortFromOutput(data);
   });
   serverProcess.on('close', (code, signal) => {
     const closeMessage = `server process process exited with code:${code} signal:${signal}`;
@@ -1227,9 +1585,14 @@ function attachServerEventHandlers(serverProcess) {
   });
 }
 if (isHeadlessMode) {
-  serverProcess = startServer(['-html-static-dir', _path.default.join(process.resourcesPath, './frontend')]);
-  attachServerEventHandlers(serverProcess);
-  _electron.shell.openExternal(`http://localhost:${defaultPort}`);
+  startServer(['-html-static-dir', _path.default.join(process.resourcesPath, './frontend')]).then(serverProcess => {
+    attachServerEventHandlers(serverProcess);
+
+    // Give 1s for backend to start
+    setTimeout(() => _electron.shell.openExternal(`http://localhost:${actualPort}`), 1000);
+  });
 } else {
-  startElecron();
+  if (!isRunningScript) {
+    startElecron();
+  }
 }
\ No newline at end of file
